@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 ########################################################################
 #
 #       License: MIT
@@ -17,7 +18,13 @@ except ImportError:
 from blosc import blosc_extension as _ext
 import blosc
 
-if sys.version_info[0] < 3:
+# version number hack
+vi = sys.version_info
+PY26 = vi[0] == 2 and vi[1] == 6
+PY27 = vi[0] == 2 and vi[1] == 7
+PY3X = vi[0] == 3
+
+if vi[0] < 3:
     int_ = (int, long)
 else:
     int_ = (int,)
@@ -111,34 +118,54 @@ def set_blocksize(blocksize):
     --------
 
     >>> blosc.set_blocksize(512)
+    >>> blosc.set_blocksize(0)
 
     """
 
     _ext.set_blocksize(blocksize)
-    
-def set_releasegil( gilstate ):
+
+
+def get_blocksize():
     """
-    set_releasegil( gitstate )
-    
-    Sets a boolean on whether to release the Python global inter-lock (GIL) 
-    during c-blosc compress and decompress operations or not.  This defaults 
-    to False.
-    
-    Notes
-    -----
-    
-    Designed to be used with larger chunk sizes and a ThreadPool.  There is a 
-    small performance penalty with releasing the GIL that will more harshly 
-    penalize small block sizes.
-    
+    get_blocksize()
+
+    Get the blocksize currently used.
+
     Examples
     --------
-    
+
+    >>> blosc.set_blocksize(0)
+    >>> blosc.get_blocksize()
+    0
+
+    """
+    return _ext.get_blocksize()
+
+
+def set_releasegil(gilstate):
+    """
+    set_releasegil( gitstate )
+
+    Sets a boolean on whether to release the Python global inter-lock (GIL)
+    during c-blosc compress and decompress operations or not.  This defaults
+    to False.
+
+    Notes
+    -----
+
+    Designed to be used with larger chunk sizes and a ThreadPool.  There is a
+    small performance penalty with releasing the GIL that will more harshly
+    penalize small block sizes.
+
+    Examples
+    --------
+
     >>> oldReleaseState = blosc.set_releasegil(True)
-    
+
     """
     gilstate = bool(gilstate)
-    return _ext.set_releasegil( gilstate )
+    return _ext.set_releasegil(gilstate)
+
 
 def compressor_list():
     """
@@ -236,6 +263,65 @@ def get_clib(bytesobj):
     return _ext.get_clib(bytesobj)
 
 
+def get_cbuffer_sizes(bytesobj):
+    """
+    get_cbuffer_sizes(bytesobj)
+
+    Return information about a compressed buffer:
+    (number of uncompressed bytes, number of compressed bytes, blocksize).
+
+    Parameters
+    ----------
+    bytesobj : str / bytes
+        The compressed buffer.
+
+    Returns
+    -------
+    out : tuple
+        The associated uncompressed bytes, compressed bytes and blocksize
+
+    Examples
+    --------
+
+    >>> s = b'0123456789'
+    >>> c = blosc.compress(s, typesize=1)
+    >>> blosc.get_cbuffer_sizes(c)
+    (10, 26, 10)
+    """
+
+    return _ext.get_cbuffer_sizes(bytesobj)
+
+
+def cbuffer_validate(bytesobj):
+    """
+    validate_cbuffer(bytesobj)
+
+    Validate the cbuffer. Check that the cbuffer is safe to compress. Note
+    that this does not guarantee that the blosc will be able to decompress the
+    buffer successfully, only that it is safe to attempt to do so.
+
+    Parameters
+    ----------
+    bytesobj : str / bytes
+        The compressed buffer.
+
+    Returns
+    -------
+    result : boolean
+        True if safe, False if not.
+
+    Examples
+    --------
+
+    >>> s = b'0123456789'
+    >>> c = blosc.compress(s, typesize=1)
+    >>> blosc.cbuffer_validate(c)
+    True
+    """
+
+    return _ext.cbuffer_validate(bytesobj)
+
+
 def free_resources():
     """
     free_resources()
@@ -613,8 +699,7 @@ def pack_array(array, clevel=9, shuffle=blosc.SHUFFLE, cname='blosclz'):
     TypeError
         If array does not quack like a numpy ndarray.
     ValueError
-        If array.itemsize * array.size is larger than the maximum allowed
-            buffer size.
+        If array.itemsize * array.size is larger than the maximum allowed buffer size.
         If typesize is not within the allowed range.
         If clevel is not within the allowed range.
         If cname is not within the supported compressors.
@@ -649,7 +734,7 @@ def pack_array(array, clevel=9, shuffle=blosc.SHUFFLE, cname='blosclz'):
     return packed_array
 
 
-def unpack_array(packed_array):
+def unpack_array(packed_array, **kwargs):
     """unpack_array(packed_array)
 
     Unpack (decompress) a packed NumPy array.
@@ -659,6 +744,10 @@ def unpack_array(packed_array):
     packed_array : str / bytes
         The packed array to be decompressed.
 
+    **kwargs : fix_imports / encoding / errors
+        Optional parameters that can be passed to the pickle.loads API
+        https://docs.python.org/3/library/pickle.html#pickle.loads
+
     Returns
     -------
     out : ndarray
@@ -680,7 +769,11 @@ def unpack_array(packed_array):
     >>> a2 = blosc.unpack_array(parray)
     >>> numpy.alltrue(a == a2)
     True
-
+    >>> a = numpy.array(['å', 'ç', 'ø'])
+    >>> parray = blosc.pack_array(a)
+    >>> a2 = blosc.unpack_array(parray)
+    >>> numpy.alltrue(a == a2)
+    True
     """
 
     _check_bytesobj(packed_array)
@@ -688,7 +781,14 @@ def unpack_array(packed_array):
     # First decompress the pickle
     pickled_array = _ext.decompress(packed_array, False)
     # ... and unpickle
-    array = pickle.loads(pickled_array)
+
+    if kwargs and PY3X:
+        array = pickle.loads(pickled_array, **kwargs)
+        if all(isinstance(x, bytes) for x in array.tolist()):
+            import numpy
+            array = numpy.array([x.decode('utf-8') for x in array.tolist()])
+    else:
+        array = pickle.loads(pickled_array)
 
     return array
 
@@ -700,6 +800,20 @@ def load_tests(loader, tests, pattern):
     tests.addTests(doctest.DocTestSuite())
     return tests
 
+def os_release_pretty_name():
+    for p in ('/etc/os-release', '/usr/lib/os-release'):
+        try:
+            f = open(p, 'rt')
+            for line in f:
+                name, _, value = line.rstrip().partition('=')
+                if name == 'PRETTY_NAME':
+                    if len(value) >= 2 and value[0] in '"\'' and value[0] == value[-1]:
+                        value = value[1:-1]
+                    return value
+        except IOError:
+            pass
+    else:
+        return None
 
 def print_versions():
     """Print all the versions of software that python-blosc relies on."""
@@ -715,7 +829,9 @@ def print_versions():
     (sysname, nodename, release, version, machine, processor) = platform.uname()
     print("Platform: %s-%s-%s (%s)" % (sysname, release, machine, version))
     if sysname == "Linux":
-        print("Linux dist: %s" % " ".join(platform.linux_distribution()[:-1]))
+        distro = os_release_pretty_name()
+        if distro:
+            print("Linux dist:", distro)
     if not processor:
         processor = "not recognized"
     print("Processor: %s" % processor)
